@@ -52,7 +52,7 @@ package com.openexchange.pop3.connect;
 import static com.openexchange.pop3.util.POP3StorageUtil.parseLoginDelaySeconds;
 import java.net.InetAddress;
 import java.util.concurrent.Callable;
-import javax.mail.internet.IDNA;
+import javax.mail.internet.idn.IDNA;
 import com.openexchange.exception.OXException;
 import com.openexchange.mail.api.IMailFolderStorage;
 import com.openexchange.pop3.POP3Access;
@@ -105,7 +105,8 @@ public final class POP3SyncMessagesCallable implements Callable<Object> {
          * Is it allowed to connect to real POP3 account to synchronize messages?
          */
         final long refreshRate = getRefreshRateMillis();
-        if (isConnectable(refreshRate)) {
+        final Long lastAccessed = getLastAccessed();
+        if (isConnectable(refreshRate, lastAccessed)) {
             final String server;
             /*
              * Check refresh rate setting
@@ -159,7 +160,7 @@ public final class POP3SyncMessagesCallable implements Callable<Object> {
                 /*
                  * Access POP3 account and synchronize
                  */
-                pop3Storage.syncMessages(isExpungeOnQuit());
+                pop3Storage.syncMessages(isExpungeOnQuit(), lastAccessed);
                 /*
                  * Update last-accessed time stamp
                  */
@@ -178,8 +179,7 @@ public final class POP3SyncMessagesCallable implements Callable<Object> {
         return null;
     }
 
-    private boolean isConnectable(final long refreshRateMillis) throws OXException {
-        final Long lastAccessed = getLastAccessed();
+    private boolean isConnectable(final long refreshRateMillis, final Long lastAccessed) throws OXException {
         return ((null == lastAccessed) || ((System.currentTimeMillis() - lastAccessed.longValue()) >= refreshRateMillis));
     }
 
