@@ -49,11 +49,15 @@
 
 package com.openexchange.pop3.storage.mailaccount;
 
+import static com.openexchange.tools.sql.DBUtils.closeSQLStuff;
 import gnu.trove.map.TIntObjectMap;
 import gnu.trove.map.hash.TIntObjectHashMap;
 import java.io.IOException;
 import java.lang.reflect.Field;
 import java.security.SecureRandom;
+import java.sql.Connection;
+import java.sql.PreparedStatement;
+import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -68,6 +72,7 @@ import javax.mail.Message;
 import javax.mail.MessagingException;
 import javax.mail.UIDFolder;
 import javax.mail.internet.MimeMessage;
+import com.openexchange.databaseold.Database;
 import com.openexchange.exception.OXException;
 import com.openexchange.mail.MailExceptionCode;
 import com.openexchange.mail.MailServletInterface;
@@ -114,21 +119,13 @@ public class MailAccountPOP3Storage implements POP3Storage {
      */
 
     private final POP3StorageProperties properties;
-
     private String path;
-
     private final POP3Access pop3Access;
-
     private MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> defaultMailAccess;
-
     private final int pop3AccountId;
-
     private MailAccountPOP3MessageStorage messageStorage;
-
     private MailAccountPOP3FolderStorage folderStorage;
-
     private char separator;
-
     private final Collection<OXException> warnings;
 
     MailAccountPOP3Storage(final POP3Access pop3Access, final POP3StorageProperties properties) throws OXException {
@@ -167,9 +164,10 @@ public class MailAccountPOP3Storage implements POP3Storage {
     }
 
     private String composeUniquePath(final int pop3AccountId, final int user, final int cid) throws OXException {
-        final MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> defaultMailAccess = getDefaultMailAccess();
-        defaultMailAccess.connect(false);
+        MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> defaultMailAccess = null;
         try {
+            defaultMailAccess = getDefaultMailAccess();
+            defaultMailAccess.connect(false);
             final String trashFullname = defaultMailAccess.getFolderStorage().getTrashFolder();
             final char sep = defaultMailAccess.getFolderStorage().getFolder("INBOX").getSeparator();
             /*
@@ -212,7 +210,9 @@ public class MailAccountPOP3Storage implements POP3Storage {
              */
             return fullname;
         } finally {
-            defaultMailAccess.close(true);
+            if (null != defaultMailAccess) {
+                defaultMailAccess.close(true);
+            }
         }
     }
 
@@ -327,9 +327,10 @@ public class MailAccountPOP3Storage implements POP3Storage {
 
     @Override
     public void connect() throws OXException {
-        final MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> defaultMailAccess = getDefaultMailAccess();
-        defaultMailAccess.connect(false);
+        MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> defaultMailAccess = null;
         try {
+            defaultMailAccess = getDefaultMailAccess();
+            defaultMailAccess.connect(false);
             // Check path existence
             final IMailFolderStorage fs = defaultMailAccess.getFolderStorage();
             if (!fs.exists(path)) {
@@ -421,13 +422,17 @@ public class MailAccountPOP3Storage implements POP3Storage {
             /*
              * Close on error
              */
-            defaultMailAccess.close(true);
+            if (null != defaultMailAccess) {
+                defaultMailAccess.close(true);
+            }
             throw e;
         } catch (final Exception e) {
             /*
              * Close on error
              */
-            defaultMailAccess.close(true);
+            if (null != defaultMailAccess) {
+                defaultMailAccess.close(true);
+            }
             throw MailExceptionCode.UNEXPECTED_ERROR.create(e, e.getMessage());
         }
     }
@@ -495,15 +500,62 @@ public class MailAccountPOP3Storage implements POP3Storage {
     };
 
     @Override
-    public void syncMessages(final boolean expunge) throws OXException {
+    public void syncMessages(final boolean expunge, final Long lastAccessed) throws OXException {
+        // Synchronize access
+        final Session session = pop3Access.getSession();
+        {
+            final int cid = session.getContextId();
+            final int user = session.getUserId();
+            final int accountId = pop3Access.getAccountId();
+            final Connection con = Database.get(cid, true);
+            PreparedStatement stmt = null;
+            try {
+                int pos = 1;
+                if (null == lastAccessed) {
+                    stmt = con.prepareStatement("INSERT INTO user_mail_account_properties (id,cid,user,name,value) VALUES (?,?,?,?,?)");
+                    stmt.setInt(pos++, accountId);
+                    stmt.setInt(pos++, cid);
+                    stmt.setInt(pos++, user);
+                    stmt.setString(pos++, POP3StoragePropertyNames.PROPERTY_LAST_ACCESSED);
+                    stmt.setString(pos++, "0"); // Dummy last-accessed
+                    // Uniquely INSERT
+                    try {
+                        final int result = stmt.executeUpdate();
+                        if (result <= 0) {
+                            // Another thread is already in process.
+                            return;
+                        }
+                    } catch (SQLException e) {
+                        // INSERT failed. Another thread is already in process.
+                        return;
+                    }
+                } else {
+                    stmt = con.prepareStatement("UPDATE user_mail_account_properties SET value=? WHERE id=? AND cid=? AND user=? AND name=? AND value=?");
+                    stmt.setString(pos++, Long.toString(lastAccessed.longValue() + 1));
+                    stmt.setInt(pos++, accountId);
+                    stmt.setInt(pos++, cid);
+                    stmt.setInt(pos++, user);
+                    stmt.setString(pos++, POP3StoragePropertyNames.PROPERTY_LAST_ACCESSED);
+                    stmt.setString(pos++, lastAccessed.toString());
+                    final int result = stmt.executeUpdate();
+                    if (result <= 0) {
+                        // Another thread is already in process.
+                        return;
+                    }
+                }
+            } catch (SQLException e) {
+                // Concurrency check failed
+                throw POP3ExceptionCode.SQL_ERROR.create(e, e.getMessage());
+            } finally {
+                closeSQLStuff(null, stmt);
+                Database.back(cid, true, con);
+            }
+        }
+        // Start sync process
         POP3Store pop3Store = null;
         try {
-            final POP3StoreResult result = POP3StoreConnector.getPOP3Store(
-                pop3Access.getPOP3Config(),
-                pop3Access.getMailProperties(),
-                false,
-                pop3Access.getSession(),
-                !expunge);
+            final POP3StoreResult result =
+                POP3StoreConnector.getPOP3Store(pop3Access.getPOP3Config(), pop3Access.getMailProperties(), false, session, !expunge);
             pop3Store = result.getPop3Store();
             final boolean containsWarnings = result.containsWarnings();
             if (containsWarnings) {
@@ -581,14 +633,16 @@ public class MailAccountPOP3Storage implements POP3Storage {
                          */
                         final Message[] messages = inbox.getMessages();
                         final Set<String> trashedUIDLs = getTrashContainer().getUIDLs();
-                        for (int i = 0; i < messages.length; i++) {
-                            final Message message = messages[i];
-                            final String uidl = seqnum2uidl.get(message.getMessageNumber());
-                            if (trashedUIDLs.contains(uidl)) {
-                                message.setFlags(FLAGS_DELETED, true);
+                        if (!trashedUIDLs.isEmpty()) {
+                            for (int i = 0; i < messages.length; i++) {
+                                final Message message = messages[i];
+                                final String uidl = seqnum2uidl.get(message.getMessageNumber());
+                                if (trashedUIDLs.contains(uidl)) {
+                                    message.setFlags(FLAGS_DELETED, true);
+                                    doExpunge = true;
+                                }
                             }
                         }
-                        doExpunge = true;
                     }
                 }
             } finally {
@@ -614,7 +668,7 @@ public class MailAccountPOP3Storage implements POP3Storage {
                 warnings.add(MailExceptionCode.IO_ERROR.create(nested, nested.getMessage()));
             } else {
                 LOG.warn("Connect to POP3 account failed: " + e.getMessage(), e);
-                warnings.add(MimeMailException.handleMessagingException(e, pop3Access.getPOP3Config(), pop3Access.getSession()));
+                warnings.add(MimeMailException.handleMessagingException(e, pop3Access.getPOP3Config(), session));
             }
         } catch (final OXException e) {
             if (MimeMailExceptionCode.LOGIN_FAILED.equals(e) || MimeMailExceptionCode.INVALID_CREDENTIALS.equals(e)) {
@@ -635,7 +689,7 @@ public class MailAccountPOP3Storage implements POP3Storage {
 
     /**
      * Checks if delete-write-through option is enabled.
-     * 
+     *
      * @return <code>true</code> if enabled; otherwise <code>false</code>
      * @throws OXException If option cannot be determined
      */
@@ -825,12 +879,28 @@ public class MailAccountPOP3Storage implements POP3Storage {
         return subarray;
     }
 
+    private static final Field messageCacheField;
+    static {
+        Field f;
+        try {
+            f = POP3Folder.class.getDeclaredField("message_cache");
+            f.setAccessible(true);
+        } catch (SecurityException e) {
+            f = null;
+        } catch (NoSuchFieldException e) {
+            f = null;
+        }
+        messageCacheField = f;
+    }
+
     @SuppressWarnings("unchecked")
     private static Vector<POP3Message> getMessageCache(final POP3Folder inbox) throws OXException {
         try {
-            final Field messageCacheField = POP3Folder.class.getDeclaredField("message_cache");
-            messageCacheField.setAccessible(true);
-            return (Vector<POP3Message>) messageCacheField.get(inbox);
+            final Field messageCacheField = MailAccountPOP3Storage.messageCacheField;
+            if (null != messageCacheField) {
+                return (Vector<POP3Message>) messageCacheField.get(inbox);
+            }
+            throw new NoSuchFieldException("message_cache");
         } catch (final SecurityException e) {
             throw MailExceptionCode.UNEXPECTED_ERROR.create(e, e.getMessage());
         } catch (final IllegalArgumentException e) {
